"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const MustacheFileGenerator_1 = require("ts-api-decorators/dist/generators/MustacheFileGenerator");
const GeneratorUtil_1 = require("ts-api-decorators/dist/Util/GeneratorUtil");
const HandlerTree_1 = require("ts-api-decorators/dist/transformer/HandlerTree");
const path = require("path");
class FunctionFileGenerator {
    constructor(opts) {
        this.opts = opts;
        this.triggers = new Map();
        this.params = new Map();
        this.mustacheGenerator = new MustacheFileGenerator_1.MustacheFileGenerator(path.join(__dirname, FunctionFileGenerator.TEMPLATE_NAME));
        for (const t of opts.triggers) {
            this.triggers.set(t.triggerMethod, t);
        }
        for (const p of opts.params) {
            this.params.set(p.paramTypeId, p);
        }
    }
    forTree(routes) {
        const handlerNodes = routes.filter(HandlerTree_1.isHandlerNode);
        return (p) => this.mustacheGenerator.generate({
            libInclude: this.getLibIncludePath(),
            scriptFiles: this.getSourceFilesForRoutes(handlerNodes, p),
            triggerType: this.getTriggerTypeForRoutes(handlerNodes).triggerType,
            route: this.getRoute(handlerNodes),
            methods: handlerNodes.map(route => route.apiMethod),
        });
    }
    getLibIncludePath() {
        return GeneratorUtil_1.GeneratorUtil.NormalizePathSlashes(this.opts.libIncludePath ? this.opts.libIncludePath : FunctionFileGenerator.LIB_INCLUDE);
    }
    getSourceFilesForRoutes(routes, p) {
        const routeArray = new Set();
        for (const route of routes) {
            const scriptPath = this.getScriptFilePathGenerator(p, route.location.file);
            if (!routeArray.has(scriptPath)) {
                routeArray.add(scriptPath);
            }
        }
        return Array.from(routeArray);
    }
    getScriptFilePathGenerator(p, routeFile) {
        return (GeneratorUtil_1.GeneratorUtil.NormalizePathSlashes(path.relative(path.dirname(p), GeneratorUtil_1.GeneratorUtil.GetTsFileOutputPath(this.opts.tsConfig, this.opts.tsConfigPath, routeFile))));
    }
    getFilenameForFunction(functionName) {
        return `${functionName}/${FunctionFileGenerator.FILE_NAME}`;
    }
    getRoute(routes) {
        if (routes.length === 0) {
            throw new Error('Must have at least one route');
        }
        const route = routes[0];
        for (let i = 1; i < routes.length; ++i) {
            if (route.route !== routes[i].route) {
                throw new Error('All routes in a function file must have the same route');
            }
        }
        return route.route;
    }
    getTriggerTypeForRoutes(routes) {
        if (routes.length === 0) {
            throw new Error('Must have at least one route');
        }
        const trigger = this.getTriggerType(routes[0]);
        for (let i = 1; i < routes.length; ++i) {
            if (trigger.triggerType !== this.getTriggerType(routes[i]).triggerType) {
                throw new Error('All routes in a function file must have the same trigger type');
            }
        }
        return trigger;
    }
    getTriggerType(route) {
        if (!this.triggers.has(route.apiMethod)) {
            throw new Error('No trigger defined for route');
        }
        return this.triggers.get(route.apiMethod);
    }
}
exports.FunctionFileGenerator = FunctionFileGenerator;
FunctionFileGenerator.LIB_INCLUDE = 'ts-api-decorators-azure-function';
FunctionFileGenerator.FILE_NAME = 'index.js';
FunctionFileGenerator.TEMPLATE_NAME = `${FunctionFileGenerator.FILE_NAME}.mustache`;
//# sourceMappingURL=FunctionFileGenerator.js.map