"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const FunctionFileGenerator_1 = require("../FunctionFileGenerator");
const HandlerTree_1 = require("ts-api-decorators/dist/transformer/HandlerTree");
const TransformerMetadata_1 = require("ts-api-decorators/dist/transformer/TransformerMetadata");
const AzFuncMetadata_1 = require("../../metadata/AzFuncMetadata");
class FunctionJsonFileGenerator {
    constructor(opts) {
        this.triggers = new Map();
        this.params = new Map();
        this.outputs = new Map();
        for (const t of opts.triggers) {
            this.triggers.set(t.triggerMethod, t);
        }
        for (const p of opts.params) {
            this.params.set(p.paramTypeId, p);
        }
        for (const p of opts.outputs) {
            this.outputs.set(p.outputTypeId, p);
        }
    }
    forTree(routes) {
        if (routes.length === 0) {
            throw new Error('Azure Function Generator requires at least one route');
        }
        return () => __awaiter(this, void 0, void 0, function* () { return Buffer.from(yield this.generateRouteStr(routes.filter(HandlerTree_1.isHandlerNode))); });
    }
    generateRouteStr(routes) {
        return __awaiter(this, void 0, void 0, function* () {
            return JSON.stringify(this.generateRoutes(routes), undefined, FunctionJsonFileGenerator.JSON_PRETTY_PRINT_SPACE);
        });
    }
    generateRoutes(routes) {
        // TODO: This whole method isn't smart enough to handle a route with multiple triggers (e.g. http and queue trigger)
        const triggers = routes.map(r => this.getTriggerType(r));
        const routePaths = new Set(routes.map(r => r.route));
        if (new Set(triggers.map(r => r.triggerType)).size !== 1) {
            throw new Error('FunctionJsonFileGenerator can only accept one trigger type per function file');
        }
        if (routePaths.size !== 1) {
            throw new Error('FunctionJsonFileGenerator can only accept one route per function file');
        }
        return {
            bindings: [
                ...triggers[0].getBindingForRoutes(routes),
                ...this.getParamBindings(routes),
            ],
            // Assumed to always be in the same directory
            scriptFile: `./${FunctionFileGenerator_1.FunctionFileGenerator.FILE_NAME}`,
        };
    }
    getParamBindings(routes) {
        const routeBindings = routes.map(route => {
            const bindings = [];
            for (const param of HandlerTree_1.WalkChildrenByType(route, HandlerTree_1.isHandlerParameterNode)) {
                if (param.paramDef.type === 6 /* Transport */) {
                    if (!this.params.has(param.paramDef.transportTypeId)) {
                        throw new Error('Param binding not registered for param: ' + param.paramDef.transportTypeId);
                    }
                    const bindingDef = this.params.get(param.paramDef.transportTypeId);
                    if (bindingDef.getBindingForParam) {
                        const binding = bindingDef.getBindingForParam(param.paramDef, route);
                        if (Array.isArray(binding)) {
                            bindings.push(...binding);
                        }
                        else if (binding) {
                            bindings.push(binding);
                        }
                    }
                }
            }
            for (const param of HandlerTree_1.WalkChildrenByType(route, HandlerTree_1.isHandlerModifierNode)) {
                const outputParamTransportType = TransformerMetadata_1.getMetadataValueByDescriptor(param.metadata, AzFuncMetadata_1.AzFuncMetadata.Output);
                if (!outputParamTransportType) {
                    continue;
                }
                if (!this.outputs.has(outputParamTransportType)) {
                    throw new Error('Output binding not registered for typeId: ' + outputParamTransportType);
                }
                const bindingDef = this.outputs.get(outputParamTransportType);
                if (bindingDef.getBindingForOutput) {
                    const binding = bindingDef.getBindingForOutput(param, route);
                    if (Array.isArray(binding)) {
                        bindings.push(...binding);
                    }
                    else if (binding) {
                        bindings.push(binding);
                    }
                }
            }
            return bindings;
        });
        const bindings = routeBindings.shift();
        for (const bind of routeBindings) {
            const routeSet = new Set(bind.concat(bindings));
            if (routeSet.size !== bindings.length) {
                throw new Error('FunctionJsonFileGenerator All routes with the same path must have the same bindings');
            }
        }
        return bindings;
    }
    getTriggerType(route) {
        if (!this.triggers.has(route.apiMethod)) {
            throw new Error('No trigger defined for route');
        }
        return this.triggers.get(route.apiMethod);
    }
    getFilenameForFunction(functionName) {
        return `${functionName}/${FunctionJsonFileGenerator.FILE_NAME}`;
    }
}
exports.FunctionJsonFileGenerator = FunctionJsonFileGenerator;
FunctionJsonFileGenerator.FILE_NAME = 'function.json';
FunctionJsonFileGenerator.JSON_PRETTY_PRINT_SPACE = 4;
//# sourceMappingURL=FunctionJsonFileGenerator.js.map